import io

import py
import pytest

import randovania.resolver.game_description
from randovania.games.prime import log_parser


def test_parse_log(test_files_dir):
    log = log_parser.parse_log(str(test_files_dir.join("echoes_log_a.txt")))
    assert log.version == "3.2"
    assert log.seed == 1145919247
    assert log.excluded_pickups == [23]
    assert log.pickup_database.entries == [
        randovania.resolver.game_description.PickupEntry("Temple Grounds", "Hive Chamber A",
                                                         "Power Bomb"),
        randovania.resolver.game_description.PickupEntry("Temple Grounds", "Hall of Honored Dead",
                                                         "Light Suit"),
        randovania.resolver.game_description.PickupEntry("Temple Grounds", "Hive Chamber B",
                                                         "Energy Tank 9"),
        randovania.resolver.game_description.PickupEntry("Temple Grounds", "War Ritual Grounds",
                                                         "Missile Expansion 18"),
        randovania.resolver.game_description.PickupEntry("Temple Grounds", "Windchamber Gateway",
                                                         "Missile Expansion 45"),
        randovania.resolver.game_description.PickupEntry("Temple Grounds", "Transport to Agon Wastes",
                                                         "Missile Expansion 37"),
        randovania.resolver.game_description.PickupEntry("Temple Grounds", "Temple Assembly Site",
                                                         "Ing Hive Key 2"),
        randovania.resolver.game_description.PickupEntry("Temple Grounds", "Grand Windchamber",
                                                         "Missile Expansion 33"),
        randovania.resolver.game_description.PickupEntry("Temple Grounds", "Dynamo Chamber",
                                                         "Missile Expansion 25"),
        randovania.resolver.game_description.PickupEntry("Temple Grounds", "Storage Cavern B",
                                                         "Energy Tank 4"),
        randovania.resolver.game_description.PickupEntry("Temple Grounds", "Plain of Dark Worship",
                                                         "Missile Expansion 29"),
        randovania.resolver.game_description.PickupEntry("Temple Grounds", "Defiled Shrine",
                                                         "Energy Tank 8"),
        randovania.resolver.game_description.PickupEntry("Temple Grounds", "Communication Area",
                                                         "Missile Expansion 6"),
        randovania.resolver.game_description.PickupEntry("Temple Grounds", "GFMC Compound",
                                                         "Energy Tank 6"),
        randovania.resolver.game_description.PickupEntry("Temple Grounds", "GFMC Compound",
                                                         "Echo Visor"),
        randovania.resolver.game_description.PickupEntry("Temple Grounds", "Accursed Lake",
                                                         "Grapple Beam"),
        randovania.resolver.game_description.PickupEntry("Temple Grounds", "Fortress Transport Access",
                                                         "Seeker Launcher"),
        randovania.resolver.game_description.PickupEntry("Temple Grounds", "Profane Path",
                                                         "Missile Expansion 4"),
        randovania.resolver.game_description.PickupEntry("Temple Grounds", "Phazon Grounds",
                                                         "Energy Tank 1"),
        randovania.resolver.game_description.PickupEntry("Temple Grounds", "Ing Reliquary",
                                                         "Missile Expansion 19"),
        randovania.resolver.game_description.PickupEntry("Great Temple", "Transport A Access",
                                                         "Power Bomb Expansion 1"),
        randovania.resolver.game_description.PickupEntry("Great Temple", "Temple Sanctuary",
                                                         "Missile Expansion 39"),
        randovania.resolver.game_description.PickupEntry("Great Temple", "Transport B Access",
                                                         "Missile Expansion 38"),
        randovania.resolver.game_description.PickupEntry("Great Temple", "Main Energy Controller",
                                                         "Violet Translator"),
        randovania.resolver.game_description.PickupEntry("Great Temple", "Main Energy Controller",
                                                         "Super Missile"),
        randovania.resolver.game_description.PickupEntry("Agon Wastes", "Mining Plaza",
                                                         "Missile Expansion 11"),
        randovania.resolver.game_description.PickupEntry("Agon Wastes", "Mining Station Access",
                                                         "Dark Suit"),
        randovania.resolver.game_description.PickupEntry("Agon Wastes", "Mining Station B",
                                                         "Cobalt Translator"),
        randovania.resolver.game_description.PickupEntry("Agon Wastes", "Transport Center",
                                                         "Boost Ball"),
        randovania.resolver.game_description.PickupEntry("Agon Wastes", "Mining Station A",
                                                         "Space Jump Boots"),
        randovania.resolver.game_description.PickupEntry("Agon Wastes", "Ing Cache 4",
                                                         "Beam Ammo Expansion 2"),
        randovania.resolver.game_description.PickupEntry("Agon Wastes", "Junction Site",
                                                         "Sky Temple Key 9"),
        randovania.resolver.game_description.PickupEntry("Agon Wastes", "Storage A", "Energy Tank 14"),
        randovania.resolver.game_description.PickupEntry("Agon Wastes", "Mine Shaft",
                                                         "Sky Temple Key 4"),
        randovania.resolver.game_description.PickupEntry("Agon Wastes", "Crossroads",
                                                         "Missile Expansion 8"),
        randovania.resolver.game_description.PickupEntry("Agon Wastes", "Sand Cache",
                                                         "Power Bomb Expansion 5"),
        randovania.resolver.game_description.PickupEntry("Agon Wastes", "Portal Access A",
                                                         "Missile Expansion 28"),
        randovania.resolver.game_description.PickupEntry("Agon Wastes", "Judgment Pit",
                                                         "Sky Temple Key 3"),
        randovania.resolver.game_description.PickupEntry("Agon Wastes", "Agon Temple",
                                                         "Missile Expansion 12"),
        randovania.resolver.game_description.PickupEntry("Agon Wastes", "Trial Tunnel",
                                                         "Power Bomb Expansion 4"),
        randovania.resolver.game_description.PickupEntry("Agon Wastes", "Central Mining Station",
                                                         "Energy Tank 12"),
        randovania.resolver.game_description.PickupEntry("Agon Wastes", "Warrior's Walk",
                                                         "Missile Expansion 43"),
        randovania.resolver.game_description.PickupEntry("Agon Wastes", "Sandcanyon",
                                                         "Missile Expansion 7"),
        randovania.resolver.game_description.PickupEntry("Agon Wastes", "Dark Agon Temple",
                                                         "Power Bomb Expansion 7"),
        randovania.resolver.game_description.PickupEntry("Agon Wastes", "Battleground",
                                                         "Sky Temple Key 1"),
        randovania.resolver.game_description.PickupEntry("Agon Wastes", "Battleground",
                                                         "Missile Expansion 1"),
        randovania.resolver.game_description.PickupEntry("Agon Wastes", "Agon Energy Controller",
                                                         "Missile Expansion 31"),
        randovania.resolver.game_description.PickupEntry("Agon Wastes", "Ventilation Area A",
                                                         "Dark Agon Key 2"),
        randovania.resolver.game_description.PickupEntry("Agon Wastes", "Command Center",
                                                         "Power Bomb Expansion 6"),
        randovania.resolver.game_description.PickupEntry("Agon Wastes", "Main Reactor",
                                                         "Missile Expansion 20"),
        randovania.resolver.game_description.PickupEntry("Agon Wastes", "Doomed Entry",
                                                         "Missile Expansion 34"),
        randovania.resolver.game_description.PickupEntry("Agon Wastes", "Sand Processing",
                                                         "Energy Tank 2"),
        randovania.resolver.game_description.PickupEntry("Agon Wastes", "Storage D",
                                                         "Missile Expansion 5"),
        randovania.resolver.game_description.PickupEntry("Agon Wastes", "Dark Oasis",
                                                         "Missile Expansion 9"),
        randovania.resolver.game_description.PickupEntry("Agon Wastes", "Storage B",
                                                         "Dark Torvus Key 2"),
        randovania.resolver.game_description.PickupEntry("Agon Wastes", "Feeding Pit",
                                                         "Missile Expansion 26"),
        randovania.resolver.game_description.PickupEntry("Agon Wastes", "Bioenergy Production",
                                                         "Amber Translator"),
        randovania.resolver.game_description.PickupEntry("Agon Wastes", "Ing Cache 1", "Spider Ball"),
        randovania.resolver.game_description.PickupEntry("Agon Wastes", "Storage C",
                                                         "Missile Expansion 2"),
        randovania.resolver.game_description.PickupEntry("Agon Wastes", "Ing Cache 2",
                                                         "Missile Expansion 32"),
        randovania.resolver.game_description.PickupEntry("Torvus Bog", "Torvus Lagoon",
                                                         "Dark Torvus Key 1"),
        randovania.resolver.game_description.PickupEntry("Torvus Bog", "Portal Chamber",
                                                         "Beam Ammo Expansion 3"),
        randovania.resolver.game_description.PickupEntry("Torvus Bog", "Path of Roots",
                                                         "Power Bomb Expansion 2"),
        randovania.resolver.game_description.PickupEntry("Torvus Bog", "Forgotten Bridge",
                                                         "Energy Tank 10"),
        randovania.resolver.game_description.PickupEntry("Torvus Bog", "Great Bridge",
                                                         "Sky Temple Key 2"),
        randovania.resolver.game_description.PickupEntry("Torvus Bog", "Cache A",
                                                         "Beam Ammo Expansion 4"),
        randovania.resolver.game_description.PickupEntry("Torvus Bog", "Plaza Access",
                                                         "Missile Expansion 27"),
        randovania.resolver.game_description.PickupEntry("Torvus Bog", "Abandoned Worksite", "Sunburst"),
        randovania.resolver.game_description.PickupEntry("Torvus Bog", "Poisoned Bog",
                                                         "Power Bomb Expansion 8"),
        randovania.resolver.game_description.PickupEntry("Torvus Bog", "Venomous Pond",
                                                         "Sky Temple Key 6"),
        randovania.resolver.game_description.PickupEntry("Torvus Bog", "Temple Access",
                                                         "Energy Transfer Module"),
        randovania.resolver.game_description.PickupEntry("Torvus Bog", "Torvus Plaza", "Energy Tank 5"),
        randovania.resolver.game_description.PickupEntry("Torvus Bog", "Putrid Alcove",
                                                         "Missile Expansion 30"),
        randovania.resolver.game_description.PickupEntry("Torvus Bog", "Torvus Grove",
                                                         "Sky Temple Key 7"),
        randovania.resolver.game_description.PickupEntry("Torvus Bog", "Torvus Temple",
                                                         "Missile Expansion 36"),
        randovania.resolver.game_description.PickupEntry("Torvus Bog", "Dark Torvus Arena",
                                                         "Missile Expansion 48"),
        randovania.resolver.game_description.PickupEntry("Torvus Bog", "Dark Torvus Arena",
                                                         "Sky Temple Key 8"),
        randovania.resolver.game_description.PickupEntry("Torvus Bog", "Underground Tunnel",
                                                         "Screw Attack"),
        randovania.resolver.game_description.PickupEntry("Torvus Bog", "Meditation Vista",
                                                         "Annihilator Beam"),
        randovania.resolver.game_description.PickupEntry("Torvus Bog", "Dark Torvus Temple",
                                                         "Missile Expansion 47"),
        randovania.resolver.game_description.PickupEntry("Torvus Bog", "Cache B", "Energy Tank 11"),
        randovania.resolver.game_description.PickupEntry("Torvus Bog", "Hydrodynamo Station",
                                                         "Missile Expansion 21"),
        randovania.resolver.game_description.PickupEntry("Torvus Bog", "Torvus Energy Controller",
                                                         "Morph Ball Bomb"),
        randovania.resolver.game_description.PickupEntry("Torvus Bog", "Undertemple Access",
                                                         "Dark Visor"),
        randovania.resolver.game_description.PickupEntry("Torvus Bog", "Gathering Hall",
                                                         "Missile Expansion 16"),
        randovania.resolver.game_description.PickupEntry("Torvus Bog", "Training Chamber", "Sonic Boom"),
        randovania.resolver.game_description.PickupEntry("Torvus Bog", "Sacrificial Chamber",
                                                         "Missile Expansion 46"),
        randovania.resolver.game_description.PickupEntry("Torvus Bog", "Undertemple",
                                                         "Missile Expansion 49"),
        randovania.resolver.game_description.PickupEntry("Torvus Bog", "Undertemple",
                                                         "Missile Expansion 24"),
        randovania.resolver.game_description.PickupEntry("Torvus Bog", "Transit Tunnel South",
                                                         "Dark Beam"),
        randovania.resolver.game_description.PickupEntry("Torvus Bog", "Transit Tunnel East",
                                                         "Ing Hive Key 3"),
        randovania.resolver.game_description.PickupEntry("Torvus Bog", "Dungeon", "Dark Agon Key 3"),
        randovania.resolver.game_description.PickupEntry("Torvus Bog", "Hydrochamber Storage",
                                                         "Energy Tank 7"),
        randovania.resolver.game_description.PickupEntry("Torvus Bog", "Undertransit One",
                                                         "Missile Expansion 44"),
        randovania.resolver.game_description.PickupEntry("Sanctuary Fortress", "Sanctuary Entrance",
                                                         "Gravity Boost"),
        randovania.resolver.game_description.PickupEntry("Sanctuary Fortress", "Reactor Core",
                                                         "Dark Torvus Key 3"),
        randovania.resolver.game_description.PickupEntry("Sanctuary Fortress", "Transit Station",
                                                         "Missile Expansion 14"),
        randovania.resolver.game_description.PickupEntry("Sanctuary Fortress", "Sanctuary Map Station",
                                                         "Missile Expansion 42"),
        randovania.resolver.game_description.PickupEntry("Sanctuary Fortress", "Hall of Combat Mastery",
                                                         "Darkburst"),
        randovania.resolver.game_description.PickupEntry("Sanctuary Fortress", "Main Research",
                                                         "Missile Expansion 41"),
        randovania.resolver.game_description.PickupEntry("Sanctuary Fortress", "Culling Chamber",
                                                         "Missile Expansion 23"),
        randovania.resolver.game_description.PickupEntry("Sanctuary Fortress",
                                                         "Central Area Transport West",
                                                         "Missile Expansion 3"),
        randovania.resolver.game_description.PickupEntry("Sanctuary Fortress", "Dynamo Works",
                                                         "Missile Expansion 22"),
        randovania.resolver.game_description.PickupEntry("Sanctuary Fortress", "Dynamo Works",
                                                         "Power Bomb Expansion 3"),
        randovania.resolver.game_description.PickupEntry("Sanctuary Fortress", "Hazing Cliff",
                                                         "Missile Expansion 15"),
        randovania.resolver.game_description.PickupEntry("Sanctuary Fortress", "Watch Station",
                                                         "Missile Launcher"),
        randovania.resolver.game_description.PickupEntry("Sanctuary Fortress", "Hive Dynamo Works",
                                                         "Dark Agon Key 1"),
        randovania.resolver.game_description.PickupEntry("Sanctuary Fortress", "Sentinel's Path",
                                                         "Energy Tank 13"),
        randovania.resolver.game_description.PickupEntry("Sanctuary Fortress", "Watch Station Access",
                                                         "Missile Expansion 35"),
        randovania.resolver.game_description.PickupEntry("Sanctuary Fortress", "Aerial Training Site",
                                                         "Missile Expansion 10"),
        randovania.resolver.game_description.PickupEntry("Sanctuary Fortress", "Aerial Training Site",
                                                         "Missile Expansion 40"),
        randovania.resolver.game_description.PickupEntry("Sanctuary Fortress", "Main Gyro Chamber",
                                                         "Missile Expansion 13"),
        randovania.resolver.game_description.PickupEntry("Sanctuary Fortress", "Vault", "Light Beam"),
        randovania.resolver.game_description.PickupEntry("Sanctuary Fortress", "Temple Access",
                                                         "Ing Hive Key 1"),
        randovania.resolver.game_description.PickupEntry("Sanctuary Fortress", "Hive Gyro Chamber",
                                                         "Emerald Translator"),
        randovania.resolver.game_description.PickupEntry("Sanctuary Fortress", "Hive Temple",
                                                         "Sky Temple Key 5"),
        randovania.resolver.game_description.PickupEntry("Sanctuary Fortress",
                                                         "Sanctuary Energy Controller",
                                                         "Missile Expansion 17"),
        randovania.resolver.game_description.PickupEntry("Sanctuary Fortress", "Hive Entrance",
                                                         "Energy Tank 3"),
        randovania.resolver.game_description.PickupEntry("Sanctuary Fortress", "Aerie",
                                                         "Beam Ammo Expansion 1"),
    ]


@pytest.mark.parametrize("log_name", [
    "echoes_log_a.txt",
    "echoes_log_recursion_heavy.txt",
])
def test_generator(test_files_dir, log_name):
    log = log_parser.parse_log(str(test_files_dir.join(log_name)))
    generated_log = log_parser.generate_log(log.seed, log.excluded_pickups, False)
    assert log == generated_log


@pytest.mark.parametrize("log_name", [
    "echoes_log_a.txt",
    "echoes_log_recursion_heavy.txt",
])
def test_write(test_files_dir: py.path.local, log_name):
    input_file = test_files_dir.join(log_name)
    log = log_parser.parse_log(str(input_file))
    output = io.StringIO()
    log.write(output)

    assert output.getvalue().split() == input_file.read_text("utf-8").split()
